@@ -1,3 +1,6 @@
+import json
+from sqlalchemy import Enum
+
 from alembic.config import Config
 import os
 import functools
@@ -13,19 +16,14 @@ from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import event
 from sqlalchemy.types import JSON
 
-from cyberfusion.QueueSupport.encoders import json_serialize
+from cyberfusion.QueueSupport.encoders import CustomEncoder
+from cyberfusion.QueueSupport.enums import QueueProcessStatus
 from cyberfusion.QueueSupport.settings import settings
 
 
 def set_sqlite_pragma(
     dbapi_connection: sqlite3.Connection, connection_record: _ConnectionRecord
 ) -> None:
-    """Enable foreign key support.
-
-    This is needed for cascade deletes to work.
-
-    See https://docs.sqlalchemy.org/en/13/dialects/sqlite.html#sqlite-foreign-keys
-    """
     cursor = dbapi_connection.cursor()
 
     cursor.execute("PRAGMA foreign_keys=ON")
@@ -51,7 +49,7 @@ def run_migrations() -> None:
 def make_database_session() -> Session:
     engine = create_engine(
         settings.database_path,
-        json_serializer=json_serialize,
+        json_serializer=lambda obj: json.dumps(obj, cls=CustomEncoder),
     )
 
     event.listen(engine, "connect", set_sqlite_pragma)
@@ -88,15 +86,35 @@ class Queue(BaseModel):
 
     __tablename__ = "queues"
 
+    queue_items = relationship(
+        "QueueItem",
+        back_populates="queue",
+        cascade="all, delete",
+    )
+    queue_processes = relationship(
+        "QueueProcess",
+        back_populates="queue",
+        cascade="all, delete",
+    )
+
 
 class QueueProcess(BaseModel):
     """QueueProcess model."""
 
     __tablename__ = "queue_processes"
 
-    queue_id = Column(Integer, ForeignKey("queues.id"), nullable=False)
-    queue = relationship("Queue")
+    queue_id = Column(
+        Integer, ForeignKey("queues.id", ondelete="CASCADE"), nullable=False, index=True
+    )
     preview = Column(Boolean, nullable=False)
+    status = Column(Enum(QueueProcessStatus), nullable=True)
+
+    queue = relationship("Queue", back_populates="queue_processes")
+    queue_item_outcomes = relationship(
+        "QueueItemOutcome",
+        back_populates="queue_process",
+        cascade="all, delete",
+    )
 
 
 class QueueItem(BaseModel):
@@ -104,25 +122,45 @@ class QueueItem(BaseModel):
 
     __tablename__ = "queue_items"
 
-    queue_id = Column(Integer, ForeignKey("queues.id"), nullable=False)
-    queue = relationship("Queue")
+    queue_id = Column(
+        Integer, ForeignKey("queues.id", ondelete="CASCADE"), nullable=False, index=True
+    )
     type = Column(String(length=255), nullable=False)
     reference = Column(String(length=255), nullable=True)
     hide_outcomes = Column(Boolean, nullable=False)
+    fail_silently = Column(Boolean, nullable=False)
     deduplicated = Column(Boolean, nullable=False)
     attributes = Column(JSON, nullable=False)
     traceback = Column(String(), nullable=True)
 
+    queue = relationship("Queue", back_populates="queue_items")
+    queue_item_outcomes = relationship(
+        "QueueItemOutcome",
+        back_populates="queue_item",
+        cascade="all, delete",
+    )
+
 
 class QueueItemOutcome(BaseModel):
     """QueueItemOutcome model."""
 
     __tablename__ = "queue_item_outcomes"
 
-    queue_item_id = Column(Integer, ForeignKey("queue_items.id"), nullable=False)
-    queue_item = relationship("QueueItem")
-    queue_process_id = Column(Integer, ForeignKey("queue_processes.id"), nullable=False)
-    queue_process = relationship("QueueProcess")
+    queue_item_id = Column(
+        Integer,
+        ForeignKey("queue_items.id", ondelete="CASCADE"),
+        nullable=False,
+        index=True,
+    )
+    queue_process_id = Column(
+        Integer,
+        ForeignKey("queue_processes.id", ondelete="CASCADE"),
+        nullable=False,
+        index=True,
+    )
     type = Column(String(length=255), nullable=False)
     attributes = Column(JSON, nullable=False)
     string = Column(String(length=255), nullable=False)
+
+    queue_item = relationship("QueueItem", back_populates="queue_item_outcomes")
+    queue_process = relationship("QueueProcess", back_populates="queue_item_outcomes")
